function GottaGoFaster.InitOptions()
  GottaGoFaster.LSM = LibStub:GetLibrary("LibSharedMedia-3.0");
  GottaGoFaster.LSM:Register("font", "Myriad Condensed Web", "Interface\\Addons\\GottaGoFaster\\MyriadCondensedWeb.ttf")
  local defaults = {
    profile = {
      GoldTimer = true,
      TimerType = "TrueTimerMS",
      FrameAnchor = "RIGHT",
      FrameX = -150,
      FrameY = 0,
      TimerAlign = "LEFT",
      TimerX = 30,
      TimerY = 0,
      TimerFont = "Myriad Condensed Web",
      TimerFontSize = 29,
      TimerColor = "ffffffff",
      TimerFontFlag = "OUTLINE",
      ObjectiveAlign = "LEFT",
      ObjectiveX = 0,
      ObjectiveY = -40,
      ObjectiveFont = "Myriad Condensed Web",
      ObjectiveFontSize = 21,
      ObjectiveCollapsed = false,
      ObjectiveColor = "ffffffff",
      ObjectiveCompleteColor = "ff0ff000",
      ObjectiveFontFlag = "OUTLINE",
      IncreaseColor = "ffffffff",
      AffixesColor = "ffffffff",
      LevelColor = "ffffffff",
      DeathColor = "ffffffff",
      LevelInTimer = true,
      LevelInObjectives = false,
      AffixesInObjectives = true,
      IncreaseInObjectives = true,
      DeathInObjectives = true,
      MobPoints = true,
      IndividualMobValue = true,
      UseMdt = true,
      ObjectiveCompleteInObjectives = true,
      TimerTooltip = true,
      PullCountToggle = true,
      History = {},
      DebugMode = false,
      Version = 0,
      AutoDialog = false,
      SpyHelper = true,
      BestReport = true,
    },
  }
  local options = {
    name = "GottaGoFaster",
    handler = GottaGoFaster,
    type = "group",
    args = {
      cms = {
        name = "Core Features",
        type = "group",
        args = {
          TimerType = {
            order = 1,
            type = "select",
            name = "Timer Type",
            desc = "TrueTimer is timed off the start of the dungeon using system time.\nI recommend you use this option.\nBlizzard's timer is known to bug up to 20 seconds.\nI will report details soon.",
            values = {["TrueTimerMS"] = "TrueTimer (Miliseconds)", ["TrueTimerNoMS"] = "TrueTimer (No Miliseconds)", ["Blizzard"] = "Blizzard's Timer"},
            get = GottaGoFaster.GetTimerType,
            set = GottaGoFaster.SetTimerType,
          },
          GoldTimer = {
            order = 2,
            type = "toggle",
            name = "Gold Timer",
            desc = "Toggles Gold Timer",
            get = GottaGoFaster.GetGoldTimer,
            set = GottaGoFaster.SetGoldTimer,
          },
          LevelInObjectives = {
            order = 3,
            type = "toggle",
            name = "CM Level Display (Objectives)",
            desc = "Show the current CM Level in the objectives list",
            get = GottaGoFaster.GetLevelInObjectives,
            set = GottaGoFaster.SetLevelInObjectives,
          },
          AffixesInObjectives = {
            order = 4,
            type = "toggle",
            name = "Affix Display (Objectives)",
            desc = "Show the current Affixes in the objectives list",
            get = GottaGoFaster.GetAffixesInObjectives,
            set = GottaGoFaster.SetAffixesInObjectives,
          },
          LevelInTimer = {
            order = 5,
            type = "toggle",
            name = "CM Level Display (Timer)",
            desc = "Show the current CM Level at the start of the timer",
            get = GottaGoFaster.GetLevelInTimer,
            set = GottaGoFaster.SetLevelInTimer,
          },
          IncreaseInObjectives = {
            order = 6,
            type = "toggle",
            name = "Keystone Increase Display (Objectives)",
            desc = "Bonus Keystone Time Splits",
            get = GottaGoFaster.GetIncreaseInObjectives,
            set = GottaGoFaster.SetIncreaseInObjectives,
          },
          ObjectiveCompleteInObjectives = {
            order = 7,
            type = "toggle",
            name = "CM Objective Complete Display (Objectives)",
            desc = "Show the time objectives we're completed at",
            get = GottaGoFaster.GetObjectiveCompleteInObjectives,
            set = GottaGoFaster.SetObjectiveCompleteInObjectives,
          },
          DeathInObjectives = {
            order = 8,
            type = "toggle",
            name = "Death Contribution Display (Objectives)",
            desc = "Show the amount of deaths, and time loss they created in objectives",
            get = GottaGoFaster.GetDeathInObjectives,
            set = GottaGoFaster.SetDeathInObjectives,
          },
          MobPoints = {
            order = 9,
            type = "toggle",
            name = "Mob Points",
            desc = "Show Exact Number Of Points Used To Calculate Enemy Forces Percentage, Appears Per Mob In Tooltip With IndividualMobValue",
            get = GottaGoFaster.GetMobPoints,
            set = GottaGoFaster.SetMobPoints,
          },
          IndividualMobValue = {
            order = 10,
            type = "toggle",
            name = "Individual Mob Value",
            desc = "Show In Tooltip Individual Mobs Value Towards Enemy Forces",
            get = GottaGoFaster.GetIndividualMobValue,
            set = GottaGoFaster.SetIndividualMobValue,
          },
          UseMdt = {
            order = 11,
            type = "toggle",
            name = "MDT Mob Count",
            desc = "When possible use Mythic Dungeon Tools for mob count, generally this addon has the most up to date values",
            get = GottaGoFaster.GetUseMdt,
            set = GottaGoFaster.SetUseMdt,
          },
          TimerTooltip = {
            order = 12,
            type = "toggle",
            name = "Timer Tooltip",
            desc = "Display Detailed Information About Current M+ By Hovering Over The Timer. This Also Prevents Full Clickthrough Frames (Clickthrough Requires Reload)",
            get = GottaGoFaster.GetTimerTooltip,
            set = GottaGoFaster.SetTimerTooltip,
          },
          PullCountToggle = {
            order = 13,
            type = "toggle",
            name = "Show Pull Count",
            desc = "Shows the count value of mobs in your current pull",
            get = GottaGoFaster.GetPullCountToggle,
            set = GottaGoFaster.SetPullCountToggle,
          },
          BestReport = {
            order = 14,
            type = "toggle",
            name = "Report Best Run",
            desc = "Reports Your Best Run At This Dungeon / Level Affixes At The Start Of The Run, And When You Beat This Run, New Record At The End",
            get = GottaGoFaster.GetBestReport,
            set = GottaGoFaster.SetBestReport,
          },
          Changelog = {
            order = 15,
            name = "Changelog",
            desc = "GottaGoFaster Changelog",
            type = "execute",
            func = GottaGoFaster.Changelog,
          },
          History = {
            order = 16,
            name = "History",
            desc = "GottaGoFaster History - Display Of Your Runs",
            type = "execute",
            func = GottaGoFaster.History,
          },
        },
      },
      displayFrame = {
        name = "Display: Frame",
        type = "group",
        args = {
          unlocked = {
            order = 1,
            type = "toggle",
            name = "Unlocked",
            desc = "Toggles Unlock State Of Timer Frame",
            get = GottaGoFaster.GetUnlocked,
            set = GottaGoFaster.SetUnlocked,
          },
          ObjectiveCollapsed = {
            order = 2,
            type = "toggle",
            name = "Objective Tracker Collapse",
            desc = "Collapse Objective Tracker When Leaving CM / Timewalker",
            get = GottaGoFaster.GetObjectiveCollapsed,
            set = GottaGoFaster.SetObjectiveCollapsed,
          },
          TimerX = {
            order = 3,
            type = "range",
            name = "Timer X Offset",
            desc = "Default: 0",
            min = -100,
            max = 100,
            step = 1,
            get = GottaGoFaster.GetTimerX,
            set = GottaGoFaster.SetTimerX,
          },
          TimerY = {
            order = 4,
            type = "range",
            name = "Timer Y Offset",
            desc = "Default: 0",
            min = -100,
            max = 100,
            step = 1,
            get = GottaGoFaster.GetTimerY,
            set = GottaGoFaster.SetTimerY,
          },
          ObjectiveX = {
            order = 5,
            type = "range",
            name = "Objective X Offset",
            desc = "Default: 0",
            min = -100,
            max = 100,
            step = 1,
            get = GottaGoFaster.GetObjectiveX,
            set = GottaGoFaster.SetObjectiveX,
          },
          ObjectiveY = {
            order = 6,
            type = "range",
            name = "Objective Y Offset",
            desc = "Default: -40",
            min = -100,
            max = 100,
            step = 1,
            get = GottaGoFaster.GetObjectiveY,
            set = GottaGoFaster.SetObjectiveY,
          },
          DemoMode = {
            order = 19,
            type = "execute",
            name = "Demo Mode",
            desc = "Shows GottaGoFaster Outside CM For Demo / Setup Purposes",
            func = GottaGoFaster.ToggleDemoMode,
          },
        },
      },
      displayFont = {
        name = "Display: Font",
        type = "group",
        args = {
          TimerFontSize = {
            order = 7,
            type = "range",
            name = "Timer Font Size",
            desc = "Default: 29",
            min = 8,
            max = 32,
            step = 1,
            get = GottaGoFaster.GetTimerFontSize,
            set = GottaGoFaster.SetTimerFontSize,
          },
          ObjectiveFontSize = {
            order = 8,
            type = "range",
            name = "Objective Font Size",
            desc = "Default: 21",
            min = 8,
            max = 32,
            step = 1,
            get = GottaGoFaster.GetObjectiveFontSize,
            set = GottaGoFaster.SetObjectiveFontSize,
          },
          TimerFontFlag = {
            order = 9,
            type = "select",
            name = "Timer Font Flag",
            desc = "Default: OUTLINE",
            values = {["OUTLINE"] = "OUTLINE", ["THICKOUTLINE"] = "THICKOUTLINE", ["MONOCHROME"] = "MONOCHROME", ["NONE"] = "NONE"},
            get = GottaGoFaster.GetTimerFontFlag,
            set = GottaGoFaster.SetTimerFontFlag,
          },
          ObjectiveFontFlag = {
            order = 10,
            type = "select",
            name = "Objective Font Flag",
            desc = "Default: OUTLINE",
            values = {["OUTLINE"] = "OUTLINE", ["THICKOUTLINE"] = "THICKOUTLINE", ["MONOCHROME"] = "MONOCHROME", ["NONE"] = "NONE"},
            get = GottaGoFaster.GetObjectiveFontFlag,
            set = GottaGoFaster.SetObjectiveFontFlag,
          },
          TimerAlign = {
            order = 11,
            type = "select",
            name = "Timer Align",
            desc = "Default: CENTER",
            values = {["LEFT"] = "LEFT", ["CENTER"] = "CENTER", ["RIGHT"] = "RIGHT"},
            get = GottaGoFaster.GetTimerAlign,
            set = GottaGoFaster.SetTimerAlign,
          },
          ObjectiveAlign = {
            order = 12,
            type = "select",
            name = "Objective Align",
            desc = "Default: LEFT",
            values = {["LEFT"] = "LEFT", ["CENTER"] = "CENTER", ["RIGHT"] = "RIGHT"},
            get = GottaGoFaster.GetObjectiveAlign,
            set = GottaGoFaster.SetObjectiveAlign,
          },
          TimerFont = {
            order = 13,
            type = "select",
            dialogControl = "LSM30_Font",
            name = "Timer Font",
            desc = "Default: Arial, Monospaced Fonts like MyriadCondensedWeb are recommended",
            values = GottaGoFaster.LSM:HashTable("font"),
            get = GottaGoFaster.GetTimerFont,
            set = GottaGoFaster.SetTimerFont,
          },
          ObjectiveFont = {
            order = 14,
            type = "select",
            dialogControl = "LSM30_Font",
            name = "Objective Font",
            desc = "Default: Arial, Monospaced Fonts like MyriadCondensedWeb are recommended",
            values = GottaGoFaster.LSM:HashTable("font"),
            get = GottaGoFaster.GetObjectiveFont,
            set = GottaGoFaster.SetObjectiveFont,
          },
          ObjectiveColor = {
            order = 15,
            type = "color",
            name = "Objective Color",
            desc = "Default: White",
            get = GottaGoFaster.GetObjectiveColor,
            set = GottaGoFaster.SetObjectiveColor,
            hasAlpha = false,
          },
          ObjectiveCompleteColor = {
            order = 16,
            type = "color",
            name = "Objective Complete Color",
            desc = "Default: Green",
            get = GottaGoFaster.GetObjectiveCompleteColor,
            set = GottaGoFaster.SetObjectiveCompleteColor,
            hasAlpha = false,
          },
          TimerColor = {
            order = 17,
            type = "color",
            name = "Timer Color",
            desc = "Default: White",
            get = GottaGoFaster.GetTimerColor,
            set = GottaGoFaster.SetTimerColor,
            hasAlpha = false,
          },
          IncreaseColor = {
            order = 18,
            type = "color",
            name = "Keystone Increase Color",
            desc = "Default: White",
            get = GottaGoFaster.GetIncreaseColor,
            set = GottaGoFaster.SetIncreaseColor,
            hasAlpha = false,
          },
          AffixesColor = {
            order = 19,
            type = "color",
            name = "Affix Information Color",
            desc = "Default: White",
            get = GottaGoFaster.GetAffixesColor,
            set = GottaGoFaster.SetAffixesColor,
            hasAlpha = false,
          },
          LevelColor = {
            order = 20,
            type = "color",
            name = "Level Information Color",
            desc = "Default: White",
            get = GottaGoFaster.GetLevelColor,
            set = GottaGoFaster.SetLevelColor,
          },
          DeathColor = {
            order = 21,
            type = "color",
            name = "Death Information Color",
            desc = "Default: White",
            get = GottaGoFaster.GetDeathColor,
            set = GottaGoFaster.SetDeathColor,
          },
        },
      },
      dungeons = {
        name = "Dungeon Features",
        type = "group",
        args = {
          AutoDialog = {
            order = 1,
            type = "toggle",
            name = "Auto Dialog",
            desc = "Automatically Accepts Dialog Boxes In M+, Warning This Applies To All CoS Items",
            get = GottaGoFaster.GetAutoDialog,
            set = GottaGoFaster.SetAutoDialog,
          },
          SpyHelper = {
            order = 2,
            type = "toggle",
            name = "CoS Spy Helper",
            desc = "Prints Text About Spy Automatically During Mini-Game",
            get = GottaGoFaster.GetSpyHelper,
            set = GottaGoFaster.SetSpyHelper,
          },
        },
      },
    },
  };
  GottaGoFaster.db = LibStub("AceDB-3.0"):New("GottaGoFasterDB", defaults, true);
  LibStub("AceConfig-3.0"):RegisterOptionsTable("GottaGoFaster", options);
  GottaGoFaster.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("GottaGoFaster", "GottaGoFaster");
end
